@@ -6,6 +6,8 @@ use gtk::glib;
 use gtk::prelude::*;
 use gtk::subclass::prelude::*;
 
+use crate::utils;
+
 mod imp {
     use std::cell::OnceCell;
 
@@ -141,51 +143,86 @@ impl Camera {
     pub(crate) fn source_element(
         &self,
         previous: Option<&gst::Element>,
-    ) -> Option<(gst::Element, gst::Element)> {
+    ) -> Result<Option<(gst::Element, gst::Element)>, glib::BoolError> {
         let device = self.device();
         let Some(previous) = previous else {
             return create_element(&device);
         };
         match device.reconfigure_element(previous) {
-            Ok(_) => None,
+            Ok(_) => Ok(None),
             Err(_) => create_element(&device),
         }
     }
 }
 
-fn create_element(device: &gst::Device) -> Option<(gst::Element, gst::Element)> {
+fn caps(device: &gst::Device) -> gst::Caps {
+    let device_caps = device
+        .caps()
+        .unwrap_or_else(|| gst_video::VideoCapsBuilder::for_encoding("video/x-raw").build());
+    utils::caps::limit_fps(&device_caps)
+}
+
+// For each resolution and format we only keep the highest resolution.
+fn filter_caps(caps: &gst::Caps) -> gst::Caps {
+    let mut best_caps = gst::Caps::new_empty();
+    caps.iter().for_each(|s| {
+        if let Some(framerate) = framerate_from_structure(s) {
+            let best = utils::caps::best_resolution_for_fps(caps, framerate);
+            best_caps.merge(best);
+        }
+    });
+
+    caps.intersect_with_mode(&best_caps, gst::CapsIntersectMode::First)
+}
+
+fn framerate_from_structure(structure: &gst::StructureRef) -> Option<gst::Fraction> {
+    // TODO Handle gst::List and gst::Array
+    if let Ok(framerate) = structure.get::<gst::Fraction>("framerate") {
+        Some(framerate)
+    } else if let Ok(range) = structure.get::<gst::FractionRange>("framerate") {
+        Some(range.max())
+    } else if let Ok(array) = structure.get::<gst::Array>("framerate") {
+        array
+            .iter()
+            .filter_map(|s| s.get::<gst::Fraction>().ok())
+            .filter(|frac| frac < &gst::Fraction::new(crate::MAXIMUM_RATE, 1))
+            .max()
+    } else if let Ok(array) = structure.get::<gst::List>("framerate") {
+        array
+            .iter()
+            .filter_map(|s| s.get::<gst::Fraction>().ok())
+            .filter(|frac| frac < &gst::Fraction::new(crate::MAXIMUM_RATE, 1))
+            .max()
+    } else {
+        None
+    }
+}
+
+fn create_element(
+    device: &gst::Device,
+) -> Result<Option<(gst::Element, gst::Element)>, glib::BoolError> {
     use gst::prelude::*;
 
     let bin = gst::Bin::new();
 
-    let device_src = device.create_element(None).ok()?;
+    let device_src = device.create_element(None)?;
     device_src.set_property("client-name", crate::APP_ID.get().unwrap());
 
+    let caps = caps(device);
+    log::debug!("Found caps: {caps}");
+    let highest_res_caps = filter_caps(&caps);
+    log::debug!("Using caps: {highest_res_caps}");
     let capsfilter = gst::ElementFactory::make("capsfilter")
-        .property(
-            "caps",
-            [
-                gst_video::VideoCapsBuilder::new().build(),
-                gst::Caps::builder("image/jpeg").build(),
-            ]
-            .into_iter()
-            .collect::<gst::Caps>(),
-        )
-        .build()
-        .unwrap();
-    let decodebin3 = gst::ElementFactory::make("decodebin3")
-        .property("caps", gst_video::VideoCapsBuilder::new().build())
-        .build()
-        .expect("Missing GStreamer Base Plug-ins");
+        .property("caps", &highest_res_caps)
+        .build()?;
+    let decodebin3 = gst::ElementFactory::make("decodebin3").build()?;
 
     let videoflip = gst::ElementFactory::make("videoflip")
         .property_from_str("video-direction", "auto")
-        .build()
-        .expect("Missing GStreamer Good Plug-ins");
+        .build()?;
 
-    bin.add_many([&device_src, &capsfilter, &decodebin3, &videoflip])
-        .unwrap();
-    gst::Element::link_many([&device_src, &capsfilter, &decodebin3]).unwrap();
+    bin.add_many([&device_src, &capsfilter, &decodebin3, &videoflip])?;
+    gst::Element::link_many([&device_src, &capsfilter, &decodebin3])?;
 
     decodebin3.connect_pad_added(glib::clone!(@weak videoflip => move |_, pad| {
         if pad.stream().is_some_and(|stream| matches!(stream.stream_type(), gst::StreamType::VIDEO)) {
@@ -195,15 +232,15 @@ fn create_element(device: &gst::Device) -> Option<(gst::Element, gst::Element)>
     }));
 
     let pad = videoflip.static_pad("src").unwrap();
-    let ghost_pad = gst::GhostPad::with_target(&pad).unwrap();
-    ghost_pad.set_active(true).unwrap();
+    let ghost_pad = gst::GhostPad::with_target(&pad)?;
+    ghost_pad.set_active(true)?;
 
-    bin.add_pad(&ghost_pad).unwrap();
+    bin.add_pad(&ghost_pad)?;
 
     let wrapper = gst::ElementFactory::make("wrappercamerabinsrc")
         .property("video-source", &bin)
         .build()
         .expect("Missing GStreamer Bad Plug-ins");
 
-    Some((wrapper, device_src))
+    Ok(Some((wrapper, device_src)))
 }
