@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 use std::cell::RefCell;
-use std::os::fd::{FromRawFd, OwnedFd};
+use std::os::fd::{AsRawFd, OwnedFd};
 use std::os::unix::io::RawFd;
 use std::sync::Once;
 
@@ -9,6 +9,8 @@ use gtk::prelude::*;
 use gtk::subclass::prelude::*;
 use gtk::{gio, glib};
 
+use crate::utils;
+
 static STARTED: Once = Once::new();
 
 mod imp {
@@ -274,16 +276,19 @@ impl DeviceProvider {
     ///
     /// One way to get a valid descriptor is with the [`org.freedesktop.portal.Camera`](https://flatpak.github.io/xdg-desktop-portal/#gdbus-org.freedesktop.portal.Camera)
     /// XDG portal, using the `OpenPipeWireRemote()` method.
-    pub fn set_fd(&self, fd: RawFd) -> Result<(), crate::PipewireError> {
+    pub fn set_fd(&self, fd: OwnedFd) -> Result<(), crate::PipewireError> {
         if STARTED.is_completed() {
             return Err(crate::PipewireError::ProvidedStarted);
         }
+        let raw_fd = fd.as_raw_fd();
         let provider = self.imp().inner.get().unwrap();
-        log::debug!("Starting device provider with file descriptor: {fd}");
+        log::debug!("Starting device provider with file descriptor: {raw_fd}");
         if provider.has_property("fd", Some(RawFd::static_type())) {
-            provider.set_property("fd", fd);
-            let owned_fd = unsafe { OwnedFd::from_raw_fd(fd) };
-            self.imp().fd.replace(Some(owned_fd));
+            provider.set_property("fd", raw_fd);
+            self.imp()
+                .fd
+                .replace(Some(fd))
+                .inspect(|old_fd| log::debug!("Freeing fd {}", old_fd.as_raw_fd()));
             Ok(())
         } else {
             log::warn!("Pipewire device provider does not have the `fd` property, please update to a version newer than 0.3.64");
@@ -333,9 +338,13 @@ impl DeviceProvider {
             gst::MessageView::DeviceAdded(e) => {
                 if let Some(s) = e.structure() {
                     if let Ok(device) = s.get::<gst::Device>("device") {
-                        if "Video/Source" == device.device_class().as_str() {
+                        if "Video/Source" == device.device_class() {
                             let device = crate::Camera::new(&device);
                             if !imp.has_camera(&device) {
+                                // We ignore/filter IR cameras.
+                                if device.caps().as_ref().is_some_and(utils::caps::is_infrared) {
+                                    return;
+                                }
                                 log::debug!(
                                     "Camera added: {}, target-object: {:?}\nProperties {:#?}",
                                     device.display_name(),
@@ -351,7 +360,7 @@ impl DeviceProvider {
             gst::MessageView::DeviceRemoved(e) => {
                 if let Some(s) = e.structure() {
                     if let Ok(device) = s.get::<gst::Device>("device") {
-                        if "Video/Source" == device.device_class().as_str() {
+                        if "Video/Source" == device.device_class() {
                             let n_items = self.n_items();
                             for n in 0..n_items {
                                 if let Some(nth_device) = self.camera(n) {
