@@ -90,7 +90,7 @@ mod imp {
                 if imp
                     .navigation_view
                     .visible_page()
-                    .is_some_and(|page| &page == &*imp.camera_page)
+                    .is_some_and(|page| page == *imp.camera_page)
                 {
                     imp.camera.stop_recording();
                     imp.recording_active.set(false);
@@ -147,6 +147,14 @@ mod imp {
                     }
                 }));
 
+            self.camera_page
+                .connect_hiding(glib::clone!(@weak obj => move |_| {
+                    obj.imp().camera.stop_stream();
+                }));
+            self.camera_page
+                .connect_showing(glib::clone!(@weak obj => move |_| {
+                    obj.imp().camera.start_stream();
+                }));
             // Load latest window state
             obj.load_window_size();
             obj.setup_gactions();
@@ -162,7 +170,7 @@ mod imp {
             self.navigation_view
                 .connect_visible_page_notify(glib::clone!(@weak obj => move |navigation_view| {
                     let imp = obj.imp();
-                    let enabled = navigation_view.visible_page().is_some_and(|page| &page == &*imp.camera_page);
+                    let enabled = navigation_view.visible_page().is_some_and(|page| page == *imp.camera_page);
                     obj.set_shutter_enabled(enabled);
                 }));
         }
@@ -172,8 +180,11 @@ mod imp {
         fn map(&self) {
             self.parent_map();
             let camera = self.camera.get();
-            let ctx = glib::MainContext::default();
-            ctx.spawn_local(glib::clone!(@weak camera => async move {
+            glib::spawn_future_local(glib::clone!(@weak camera => async move {
+                // HACK we add a small timeout to give the shell time to get the
+                // windows focus, otherwise the Shell won't ask for camera
+                // permission.
+                glib::timeout_future(std::time::Duration::from_millis(50)).await;
                 camera.start().await;
             }));
         }
@@ -306,8 +317,7 @@ impl Window {
             duration,
             glib::clone!(@weak self as window => move || {
                 window.imp().countdown_timer_id.take();
-                let ctx = glib::MainContext::default();
-                ctx.spawn_local(glib::clone!(@weak window => async move {
+                glib::spawn_future_local(glib::clone!(@weak window => async move {
                     if let Err(err) = window.shutter_action().await {
                         match window.capture_mode() {
                             CaptureMode::Picture => {
